import { Injectable } from '@angular/core';
import { catchError, map, Observable, throwError } from 'rxjs';
import { User } from '../models/user';
import { HttpClient } from '@angular/common/http';
import { UserInfoResponse } from './responses/user';
import { environment } from '../../environment/environment';

@Injectable({
  providedIn: 'root'
})
export class UserService {
  private users: Map<string, User> = new Map<string, User>();

  constructor(private http: HttpClient) { }

  public GetUser(username: string): Observable<User> {
    if (this.users.has(username)) {
      return new Observable<User>(subscriber => {
        subscriber.next(this.users.get(username)!);
        subscriber.complete();
      });
    } else {
      let url = `${environment.apiBase}/user/info/${username}`
      return this.http.get<UserInfoResponse>(url, { withCredentials: true }).pipe(
        map(response => {
          if (response.error) {
            throw new Error(response.error);
          }

          if (response.user) {
            this.users.set(username, response.user);
            return response.user;
          }

          throw new Error("bad API response, missing user with no error");
        }),
        catchError(error => throwError(() => new Error(error.error.message)))
      );
    }
  }

  public GetProfilePictureURL(username: string): Observable<string> {
    if (this.users.has(username)) {
      return new Observable<string>(subscriber => {
        subscriber.next(this.users.get(username)!.picture);
        subscriber.complete();
      });
    } else {
      return this.GetUser(username).pipe(map(user => user.picture));
    }
  }
}
